<?php

namespace Oro\Bundle\ProductBundle\DependencyInjection;

use Oro\Bundle\ConfigBundle\Config\ConfigManager;
use Oro\Bundle\ConfigBundle\DependencyInjection\SettingsBuilder;
use Oro\Bundle\CurrencyBundle\Rounding\RoundingServiceInterface;
use Oro\Bundle\ProductBundle\Entity\Product;
use Symfony\Component\Config\Definition\Builder\TreeBuilder;
use Symfony\Component\Config\Definition\ConfigurationInterface;

class Configuration implements ConfigurationInterface
{
    const ROOT_NODE = OroProductExtension::ALIAS;
    const RELATED_PRODUCTS_ENABLED = 'related_products_enabled';
    const RELATED_PRODUCTS_BIDIRECTIONAL = 'related_products_bidirectional';
    const MAX_NUMBER_OF_RELATED_PRODUCTS = 'max_number_of_related_products';
    const MAX_NUMBER_OF_RELATED_PRODUCTS_COUNT = 25;
    const SINGLE_UNIT_MODE = 'single_unit_mode';
    const SINGLE_UNIT_MODE_SHOW_CODE = 'single_unit_mode_show_code';
    const DEFAULT_UNIT = 'default_unit';
    const PRODUCT_IMAGE_WATERMARK_FILE = 'product_image_watermark_file';
    const PRODUCT_IMAGE_WATERMARK_SIZE = 'product_image_watermark_size';
    const PRODUCT_IMAGE_WATERMARK_POSITION = 'product_image_watermark_position';
    const FEATURED_PRODUCTS_SEGMENT_ID = 'featured_products_segment_id';
    const ENABLE_QUICK_ORDER_FORM = 'enable_quick_order_form';
    const DIRECT_URL_PREFIX = 'product_direct_url_prefix';
    const BRAND_DIRECT_URL_PREFIX = 'brand_direct_url_prefix';
    const PRODUCT_COLLECTIONS_INDEXATION_CRON_SCHEDULE = 'product_collections_indexation_cron_schedule';
    const DEFAULT_CRON_SCHEDULE = '0 * * * *';
    const PRODUCT_PROMOTION_SHOW_ON_VIEW = 'product_promotion_show_on_product_view';
    const PRODUCT_COLLECTION_MASS_ACTION_LIMITATION = 'product_collections_mass_action_limitation';
    const NEW_ARRIVALS_PRODUCT_SEGMENT_ID = 'new_arrivals_products_segment_id';
    const NEW_ARRIVALS_MAX_ITEMS = 'new_arrivals_max_items';
    const NEW_ARRIVALS_MIN_ITEMS = 'new_arrivals_min_items';
    const NEW_ARRIVALS_USE_SLIDER_ON_MOBILE = 'new_arrivals_use_slider_on_mobile';

    /**
     * {@inheritDoc}
     */
    public function getConfigTreeBuilder()
    {
        $treeBuilder = new TreeBuilder();

        $rootNode = $treeBuilder->root(self::ROOT_NODE);

        SettingsBuilder::append(
            $rootNode,
            [
                self::RELATED_PRODUCTS_ENABLED => ['value' => true],
                self::RELATED_PRODUCTS_BIDIRECTIONAL => ['value' => false],
                self::MAX_NUMBER_OF_RELATED_PRODUCTS => [
                    'value' => self::MAX_NUMBER_OF_RELATED_PRODUCTS_COUNT,
                ],
                'unit_rounding_type' => ['value' => RoundingServiceInterface::ROUND_HALF_UP],
                self::SINGLE_UNIT_MODE => ['value' => false, 'type' => 'boolean'],
                self::SINGLE_UNIT_MODE_SHOW_CODE => ['value' => false, 'type' => 'boolean'],
                self::DEFAULT_UNIT => ['value' => 'each'],
                'default_unit_precision' => ['value' => 0],
                'general_frontend_product_visibility' => [
                    'value' => [
                        Product::INVENTORY_STATUS_IN_STOCK,
                        Product::INVENTORY_STATUS_OUT_OF_STOCK
                    ]
                ],
                self::PRODUCT_IMAGE_WATERMARK_FILE => ['value' => null],
                self::PRODUCT_IMAGE_WATERMARK_SIZE => ['value' => 100],
                self::PRODUCT_IMAGE_WATERMARK_POSITION => ['value' => 'center'],
                self::FEATURED_PRODUCTS_SEGMENT_ID => [
                    'value' => '@oro_product.provider.default_value.featured_products'
                ],
                self::ENABLE_QUICK_ORDER_FORM => ['type' => 'boolean', 'value' => true],
                self::DIRECT_URL_PREFIX => ['value' => ''],
                self::PRODUCT_COLLECTIONS_INDEXATION_CRON_SCHEDULE => ['value' => self::DEFAULT_CRON_SCHEDULE],
                self::PRODUCT_PROMOTION_SHOW_ON_VIEW => ['value' => false, 'type' => 'boolean'],
                self::BRAND_DIRECT_URL_PREFIX => ['value' => ''],
                self::PRODUCT_COLLECTION_MASS_ACTION_LIMITATION => ['value' => 500],
                self::NEW_ARRIVALS_PRODUCT_SEGMENT_ID => [
                    'value' => '@oro_product.provider.default_value.new_arrivals'
                ],
                self::NEW_ARRIVALS_MAX_ITEMS => ['type' => 'integer', 'value' => 4],
                self::NEW_ARRIVALS_MIN_ITEMS => ['type' => 'integer', 'value' => 3],
                self::NEW_ARRIVALS_USE_SLIDER_ON_MOBILE => ['type' => 'boolean', 'value' => false],
            ]
        );

        return $treeBuilder;
    }


    /**
     * @param string $key
     * @return string
     */
    public static function getConfigKeyByName($key)
    {
        return implode(ConfigManager::SECTION_MODEL_SEPARATOR, [self::ROOT_NODE, $key]);
    }
}
